@@ -1,26 +1,45 @@
 /*
  * fsm.c - {Link, IP} Control Protocol Finite State Machine.
  *
- * Copyright (c) 1989 Carnegie Mellon University.
- * All rights reserved.
+ * Copyright (c) 1984-2000 Carnegie Mellon University. All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation,
- * advertising materials, and other materials related to such
- * distribution and use acknowledge that the software was developed
- * by Carnegie Mellon University.  The name of the
- * University may not be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any legal
+ *    details, please contact
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: fsm.c,v 1.8 1994/11/10 01:52:05 paulus Exp $";
-#endif
-
 /*
  * TODO:
  * Randomize fsm id on link/init.
@@ -30,21 +49,19 @@ static char rcsid[] = "$Id: fsm.c,v 1.8 1994/11/10 01:52:05 paulus Exp $";
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
-#include <syslog.h>
 
 #include "pppd.h"
 #include "fsm.h"
 
-extern char *proto_name();
 
-static void fsm_timeout __P((caddr_t));
-static void fsm_rconfreq __P((fsm *, int, u_char *, int));
-static void fsm_rconfack __P((fsm *, int, u_char *, int));
-static void fsm_rconfnakrej __P((fsm *, int, int, u_char *, int));
-static void fsm_rtermreq __P((fsm *, int));
-static void fsm_rtermack __P((fsm *));
-static void fsm_rcoderej __P((fsm *, u_char *, int));
-static void fsm_sconfreq __P((fsm *, int));
+static void fsm_timeout (void *);
+static void fsm_rconfreq (fsm *, int, u_char *, int);
+static void fsm_rconfack (fsm *, int, u_char *, int);
+static void fsm_rconfnakrej (fsm *, int, int, u_char *, int);
+static void fsm_rtermreq (fsm *, int, u_char *, int);
+static void fsm_rtermack (fsm *);
+static void fsm_rcoderej (fsm *, u_char *, int);
+static void fsm_sconfreq (fsm *, int);
 
 #define PROTO_NAME(f)	((f)->callbacks->proto_name)
 
@@ -57,8 +74,7 @@ int peer_mru[NUM_PPP];
  * Initialize fsm state.
  */
 void
-fsm_init(f)
-    fsm *f;
+fsm_init(fsm *f)
 {
     f->state = INITIAL;
     f->flags = 0;
@@ -67,6 +83,7 @@ fsm_init(f)
     f->maxconfreqtransmits = DEFMAXCONFREQS;
     f->maxtermtransmits = DEFMAXTERMREQS;
     f->maxnakloops = DEFMAXNAKLOOPS;
+    f->term_reason_len = 0;
 }
 
 
@@ -74,8 +91,7 @@ fsm_init(f)
  * fsm_lowerup - The lower layer is up.
  */
 void
-fsm_lowerup(f)
-    fsm *f;
+fsm_lowerup(fsm *f)
 {
     switch( f->state ){
     case INITIAL:
@@ -93,8 +109,7 @@ fsm_lowerup(f)
 	break;
 
     default:
-	FSMDEBUG((LOG_INFO, "%s: Up event in state %d!",
-		  PROTO_NAME(f), f->state));
+	FSMDEBUG(("%s: Up event in state %d!", PROTO_NAME(f), f->state));
     }
 }
 
@@ -105,8 +120,7 @@ fsm_lowerup(f)
  * Cancel all timeouts and inform upper layers.
  */
 void
-fsm_lowerdown(f)
-    fsm *f;
+fsm_lowerdown(fsm *f)
 {
     switch( f->state ){
     case CLOSED:
@@ -121,7 +135,7 @@ fsm_lowerdown(f)
 
     case CLOSING:
 	f->state = INITIAL;
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	break;
 
     case STOPPING:
@@ -129,7 +143,7 @@ fsm_lowerdown(f)
     case ACKRCVD:
     case ACKSENT:
 	f->state = STARTING;
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	break;
 
     case OPENED:
@@ -139,8 +153,7 @@ fsm_lowerdown(f)
 	break;
 
     default:
-	FSMDEBUG((LOG_INFO, "%s: Down event in state %d!",
-		  PROTO_NAME(f), f->state));
+	FSMDEBUG(("%s: Down event in state %d!", PROTO_NAME(f), f->state));
     }
 }
 
@@ -149,8 +162,7 @@ fsm_lowerdown(f)
  * fsm_open - Link is allowed to come up.
  */
 void
-fsm_open(f)
-    fsm *f;
+fsm_open(fsm *f)
 {
     switch( f->state ){
     case INITIAL:
@@ -182,6 +194,42 @@ fsm_open(f)
     }
 }
 
+/*
+ * terminate_layer - Start process of shutting down the FSM
+ *
+ * Cancel any timeout running, notify upper layers we're done, and
+ * send a terminate-request message as configured.
+ */
+static void
+terminate_layer(fsm *f, int nextstate)
+{
+    if( f->state != OPENED )
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
+    else if( f->callbacks->down )
+	(*f->callbacks->down)(f);	/* Inform upper layers we're down */
+
+    /* Init restart counter and send Terminate-Request */
+    f->retransmits = f->maxtermtransmits;
+    fsm_sdata(f, TERMREQ, f->reqid = ++f->id,
+	      (u_char *) f->term_reason, f->term_reason_len);
+
+    if (f->retransmits == 0) {
+	/*
+	 * User asked for no terminate requests at all; just close it.
+	 * We've already fired off one Terminate-Request just to be nice
+	 * to the peer, but we're not going to wait for a reply.
+	 */
+	f->state = nextstate == CLOSING ? CLOSED : STOPPED;
+	if( f->callbacks->finished )
+	    (*f->callbacks->finished)(f);
+	return;
+    }
+
+    TIMEOUT(fsm_timeout, f, f->timeouttime);
+    --f->retransmits;
+
+    f->state = nextstate;
+}
 
 /*
  * fsm_close - Start closing connection.
@@ -190,9 +238,10 @@ fsm_open(f)
  * the CLOSED state.
  */
 void
-fsm_close(f)
-    fsm *f;
+fsm_close(fsm *f, char *reason)
 {
+    f->term_reason = reason;
+    f->term_reason_len = (reason == NULL? 0: strlen(reason));
     switch( f->state ){
     case STARTING:
 	f->state = INITIAL;
@@ -208,18 +257,7 @@ fsm_close(f)
     case ACKRCVD:
     case ACKSENT:
     case OPENED:
-	if( f->state != OPENED )
-	    UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
-	else if( f->callbacks->down )
-	    (*f->callbacks->down)(f);	/* Inform upper layers we're down */
-
-	/* Init restart counter, send Terminate-Request */
-	f->retransmits = f->maxtermtransmits;
-	fsm_sdata(f, TERMREQ, f->reqid = ++f->id, NULL, 0);
-	TIMEOUT(fsm_timeout, (caddr_t) f, f->timeouttime);
-	--f->retransmits;
-
-	f->state = CLOSING;
+	terminate_layer(f, CLOSING);
 	break;
     }
 }
@@ -229,8 +267,7 @@ fsm_close(f)
  * fsm_timeout - Timeout expired.
  */
 static void
-fsm_timeout(arg)
-    caddr_t arg;
+fsm_timeout(void *arg)
 {
     fsm *f = (fsm *) arg;
 
@@ -246,8 +283,9 @@ fsm_timeout(arg)
 		(*f->callbacks->finished)(f);
 	} else {
 	    /* Send Terminate-Request */
-	    fsm_sdata(f, TERMREQ, f->reqid = ++f->id, NULL, 0);
-	    TIMEOUT(fsm_timeout, (caddr_t) f, f->timeouttime);
+	    fsm_sdata(f, TERMREQ, f->reqid = ++f->id,
+		      (u_char *) f->term_reason, f->term_reason_len);
+	    TIMEOUT(fsm_timeout, f, f->timeouttime);
 	    --f->retransmits;
 	}
 	break;
@@ -256,8 +294,7 @@ fsm_timeout(arg)
     case ACKRCVD:
     case ACKSENT:
 	if (f->retransmits <= 0) {
-	    syslog(LOG_WARNING, "%s: timeout sending Config-Requests",
-		   PROTO_NAME(f));
+	    warn("%s: timeout sending Config-Requests\n", PROTO_NAME(f));
 	    f->state = STOPPED;
 	    if( (f->flags & OPT_PASSIVE) == 0 && f->callbacks->finished )
 		(*f->callbacks->finished)(f);
@@ -273,8 +310,7 @@ fsm_timeout(arg)
 	break;
 
     default:
-	FSMDEBUG((LOG_INFO, "%s: Timeout event in state %d!",
-		  PROTO_NAME(f), f->state));
+	FSMDEBUG(("%s: Timeout event in state %d!", PROTO_NAME(f), f->state));
     }
 }
 
@@ -283,12 +319,9 @@ fsm_timeout(arg)
  * fsm_input - Input packet.
  */
 void
-fsm_input(f, inpacket, l)
-    fsm *f;
-    u_char *inpacket;
-    int l;
+fsm_input(fsm *f, u_char *inpacket, int l)
 {
-    u_char *inp, *outp;
+    u_char *inp;
     u_char code, id;
     int len;
 
@@ -298,27 +331,24 @@ fsm_input(f, inpacket, l)
      */
     inp = inpacket;
     if (l < HEADERLEN) {
-	FSMDEBUG((LOG_WARNING, "fsm_input(%x): Rcvd short header.",
-		  f->protocol));
+	FSMDEBUG(("fsm_input(%x): Rcvd short header.", f->protocol));
 	return;
     }
     GETCHAR(code, inp);
     GETCHAR(id, inp);
     GETSHORT(len, inp);
     if (len < HEADERLEN) {
-	FSMDEBUG((LOG_INFO, "fsm_input(%x): Rcvd illegal length.",
-		  f->protocol));
+	FSMDEBUG(("fsm_input(%x): Rcvd illegal length.", f->protocol));
 	return;
     }
     if (len > l) {
-	FSMDEBUG((LOG_INFO, "fsm_input(%x): Rcvd short packet.",
-		  f->protocol));
+	FSMDEBUG(("fsm_input(%x): Rcvd short packet.", f->protocol));
 	return;
     }
     len -= HEADERLEN;		/* subtract header length */
 
     if( f->state == INITIAL || f->state == STARTING ){
-	FSMDEBUG((LOG_INFO, "fsm_input(%x): Rcvd packet in state %d.",
+	FSMDEBUG(("fsm_input(%x): Rcvd packet in state %d.",
 		  f->protocol, f->state));
 	return;
     }
@@ -341,7 +371,7 @@ fsm_input(f, inpacket, l)
 	break;
     
     case TERMREQ:
-	fsm_rtermreq(f, id);
+	fsm_rtermreq(f, id, inp, len);
 	break;
     
     case TERMACK:
@@ -365,16 +395,10 @@ fsm_input(f, inpacket, l)
  * fsm_rconfreq - Receive Configure-Request.
  */
 static void
-fsm_rconfreq(f, id, inp, len)
-    fsm *f;
-    u_char id;
-    u_char *inp;
-    int len;
+fsm_rconfreq(fsm *f, int id, u_char *inp, int len)
 {
-    u_char *outp;
     int code, reject_if_disagree;
 
-    FSMDEBUG((LOG_INFO, "fsm_rconfreq(%s): Rcvd id %d.", PROTO_NAME(f), id));
     switch( f->state ){
     case CLOSED:
 	/* Go away, we're closed */
@@ -389,6 +413,7 @@ fsm_rconfreq(f, id, inp, len)
 	if( f->callbacks->down )
 	    (*f->callbacks->down)(f);	/* Inform upper layers */
 	fsm_sconfreq(f, 0);		/* Send initial Configure-Request */
+	f->state = REQSENT;
 	break;
 
     case STOPPED:
@@ -415,7 +440,7 @@ fsm_rconfreq(f, id, inp, len)
 
     if (code == CONFACK) {
 	if (f->state == ACKRCVD) {
-	    UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	    UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	    f->state = OPENED;
 	    if (f->callbacks->up)
 		(*f->callbacks->up)(f);	/* Inform upper layers */
@@ -424,7 +449,7 @@ fsm_rconfreq(f, id, inp, len)
 	f->nakloops = 0;
 
     } else {
-	/* we sent CONFACK or CONFREJ */
+	/* we sent CONFNAK or CONFREJ */
 	if (f->state != ACKRCVD)
 	    f->state = REQSENT;
 	if( code == CONFNAK )
@@ -437,25 +462,18 @@ fsm_rconfreq(f, id, inp, len)
  * fsm_rconfack - Receive Configure-Ack.
  */
 static void
-fsm_rconfack(f, id, inp, len)
-    fsm *f;
-    int id;
-    u_char *inp;
-    int len;
+fsm_rconfack(fsm *f, int id, u_char *inp, int len)
 {
-    FSMDEBUG((LOG_INFO, "fsm_rconfack(%s): Rcvd id %d.",
-	      PROTO_NAME(f), id));
-
     if (id != f->reqid || f->seen_ack)		/* Expected id? */
 	return;					/* Nope, toss... */
     if( !(f->callbacks->ackci? (*f->callbacks->ackci)(f, inp, len):
 	  (len == 0)) ){
 	/* Ack is bad - ignore it */
-	FSMDEBUG((LOG_INFO, "%s: received bad Ack (length %d)",
-		  PROTO_NAME(f), len));
+	error("Received bad configure-ack: %P", inp, len);
 	return;
     }
     f->seen_ack = 1;
+    f->rnakloops = 0;
 
     switch (f->state) {
     case CLOSED:
@@ -470,13 +488,13 @@ fsm_rconfack(f, id, inp, len)
 
     case ACKRCVD:
 	/* Huh? an extra valid Ack? oh well... */
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	fsm_sconfreq(f, 0);
 	f->state = REQSENT;
 	break;
 
     case ACKSENT:
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	f->state = OPENED;
 	f->retransmits = f->maxconfreqtransmits;
 	if (f->callbacks->up)
@@ -498,26 +516,31 @@ fsm_rconfack(f, id, inp, len)
  * fsm_rconfnakrej - Receive Configure-Nak or Configure-Reject.
  */
 static void
-fsm_rconfnakrej(f, code, id, inp, len)
-    fsm *f;
-    int code, id;
-    u_char *inp;
-    int len;
+fsm_rconfnakrej(fsm *f, int code, int id, u_char *inp, int len)
 {
-    int (*proc)();
-
-    FSMDEBUG((LOG_INFO, "fsm_rconfnakrej(%s): Rcvd id %d.",
-	      PROTO_NAME(f), id));
+    int ret;
+    int treat_as_reject;
 
     if (id != f->reqid || f->seen_ack)	/* Expected id? */
 	return;				/* Nope, toss... */
-    proc = (code == CONFNAK)? f->callbacks->nakci: f->callbacks->rejci;
-    if (!proc || !proc(f, inp, len)) {
-	/* Nak/reject is bad - ignore it */
-	FSMDEBUG((LOG_INFO, "%s: received bad %s (length %d)",
-		  PROTO_NAME(f), (code==CONFNAK? "Nak": "reject"), len));
-	return;
+
+    if (code == CONFNAK) {
+	++f->rnakloops;
+	treat_as_reject = (f->rnakloops >= f->maxnakloops);
+	if (f->callbacks->nakci == NULL
+	    || !(ret = f->callbacks->nakci(f, inp, len, treat_as_reject))) {
+	    error("Received bad configure-nak: %P", inp, len);
+	    return;
+	}
+    } else {
+	f->rnakloops = 0;
+	if (f->callbacks->rejci == NULL
+	    || !(ret = f->callbacks->rejci(f, inp, len))) {
+	    error("Received bad configure-rej: %P", inp, len);
+	    return;
+	}
     }
+
     f->seen_ack = 1;
 
     switch (f->state) {
@@ -529,13 +552,16 @@ fsm_rconfnakrej(f, code, id, inp, len)
     case REQSENT:
     case ACKSENT:
 	/* They didn't agree to what we wanted - try another request */
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
-	fsm_sconfreq(f, 0);		/* Send Configure-Request */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
+	if (ret < 0)
+	    f->state = STOPPED;		/* kludge for stopping CCP */
+	else
+	    fsm_sconfreq(f, 0);		/* Send Configure-Request */
 	break;
 
     case ACKRCVD:
 	/* Got a Nak/reject when we had already had an Ack?? oh well... */
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	fsm_sconfreq(f, 0);
 	f->state = REQSENT;
 	break;
@@ -555,13 +581,8 @@ fsm_rconfnakrej(f, code, id, inp, len)
  * fsm_rtermreq - Receive Terminate-Req.
  */
 static void
-fsm_rtermreq(f, id)
-    fsm *f;
-    int id;
+fsm_rtermreq(fsm *f, int id, u_char *p, int len)
 {
-    FSMDEBUG((LOG_INFO, "fsm_rtermreq(%s): Rcvd id %d.",
-	      PROTO_NAME(f), id));
-
     switch (f->state) {
     case ACKRCVD:
     case ACKSENT:
@@ -569,12 +590,15 @@ fsm_rtermreq(f, id)
 	break;
 
     case OPENED:
-	syslog(LOG_INFO, "%s terminated at peer's request", PROTO_NAME(f));
-	if (f->callbacks->down)
-	    (*f->callbacks->down)(f);	/* Inform upper layers */
+	if (len > 0) {
+	    info("%s terminated by peer (%0.*v)", PROTO_NAME(f), len, p);
+	} else
+	    info("%s terminated by peer", PROTO_NAME(f));
 	f->retransmits = 0;
 	f->state = STOPPING;
-	TIMEOUT(fsm_timeout, (caddr_t) f, f->timeouttime);
+	if (f->callbacks->down)
+	    (*f->callbacks->down)(f);	/* Inform upper layers */
+	TIMEOUT(fsm_timeout, f, f->timeouttime);
 	break;
     }
 
@@ -586,20 +610,17 @@ fsm_rtermreq(f, id)
  * fsm_rtermack - Receive Terminate-Ack.
  */
 static void
-fsm_rtermack(f)
-    fsm *f;
+fsm_rtermack(fsm *f)
 {
-    FSMDEBUG((LOG_INFO, "fsm_rtermack(%s).", PROTO_NAME(f)));
-
     switch (f->state) {
     case CLOSING:
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);
+	UNTIMEOUT(fsm_timeout, f);
 	f->state = CLOSED;
 	if( f->callbacks->finished )
 	    (*f->callbacks->finished)(f);
 	break;
     case STOPPING:
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);
+	UNTIMEOUT(fsm_timeout, f);
 	f->state = STOPPED;
 	if( f->callbacks->finished )
 	    (*f->callbacks->finished)(f);
@@ -613,6 +634,7 @@ fsm_rtermack(f)
 	if (f->callbacks->down)
 	    (*f->callbacks->down)(f);	/* Inform upper layers */
 	fsm_sconfreq(f, 0);
+	f->state = REQSENT;
 	break;
     }
 }
@@ -622,23 +644,17 @@ fsm_rtermack(f)
  * fsm_rcoderej - Receive an Code-Reject.
  */
 static void
-fsm_rcoderej(f, inp, len)
-    fsm *f;
-    u_char *inp;
-    int len;
+fsm_rcoderej(fsm *f, u_char *inp, int len)
 {
     u_char code, id;
 
-    FSMDEBUG((LOG_INFO, "fsm_rcoderej(%s).", PROTO_NAME(f)));
-
     if (len < HEADERLEN) {
-	FSMDEBUG((LOG_INFO, "fsm_rcoderej: Rcvd short Code-Reject packet!"));
+	FSMDEBUG(("fsm_rcoderej: Rcvd short Code-Reject packet!"));
 	return;
     }
     GETCHAR(code, inp);
     GETCHAR(id, inp);
-    syslog(LOG_WARNING, "%s: Rcvd Code-Reject for code %d, id %d",
-	   PROTO_NAME(f), code, id);
+    warn("%s: Rcvd Code-Reject for code %d, id %d", PROTO_NAME(f), code, id);
 
     if( f->state == ACKRCVD )
 	f->state = REQSENT;
@@ -651,12 +667,11 @@ fsm_rcoderej(f, inp, len)
  * Treat this as a catastrophic error (RXJ-).
  */
 void
-fsm_protreject(f)
-    fsm *f;
+fsm_protreject(fsm *f)
 {
     switch( f->state ){
     case CLOSING:
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	/* fall through */
     case CLOSED:
 	f->state = CLOSED;
@@ -668,7 +683,7 @@ fsm_protreject(f)
     case REQSENT:
     case ACKRCVD:
     case ACKSENT:
-	UNTIMEOUT(fsm_timeout, (caddr_t) f);	/* Cancel timeout */
+	UNTIMEOUT(fsm_timeout, f);	/* Cancel timeout */
 	/* fall through */
     case STOPPED:
 	f->state = STOPPED;
@@ -677,20 +692,11 @@ fsm_protreject(f)
 	break;
 
     case OPENED:
-	if( f->callbacks->down )
-	    (*f->callbacks->down)(f);
-
-	/* Init restart counter, send Terminate-Request */
-	f->retransmits = f->maxtermtransmits;
-	fsm_sdata(f, TERMREQ, f->reqid = ++f->id, NULL, 0);
-	TIMEOUT(fsm_timeout, (caddr_t) f, f->timeouttime);
-	--f->retransmits;
-
-	f->state = STOPPING;
+	terminate_layer(f, STOPPING);
 	break;
 
     default:
-	FSMDEBUG((LOG_INFO, "%s: Protocol-reject event in state %d!",
+	FSMDEBUG(("%s: Protocol-reject event in state %d!",
 		  PROTO_NAME(f), f->state));
     }
 }
@@ -700,18 +706,17 @@ fsm_protreject(f)
  * fsm_sconfreq - Send a Configure-Request.
  */
 static void
-fsm_sconfreq(f, retransmit)
-    fsm *f;
-    int retransmit;
+fsm_sconfreq(fsm *f, int retransmit)
 {
     u_char *outp;
-    int outlen, cilen;
+    int cilen;
 
     if( f->state != REQSENT && f->state != ACKRCVD && f->state != ACKSENT ){
 	/* Not currently negotiating - reset options */
 	if( f->callbacks->resetci )
 	    (*f->callbacks->resetci)(f);
 	f->nakloops = 0;
+	f->rnakloops = 0;
     }
 
     if( !retransmit ){
@@ -740,10 +745,7 @@ fsm_sconfreq(f, retransmit)
 
     /* start the retransmit timer */
     --f->retransmits;
-    TIMEOUT(fsm_timeout, (caddr_t) f, f->timeouttime);
-
-    FSMDEBUG((LOG_INFO, "%s: sending Configure-Request, id %d",
-	      PROTO_NAME(f), f->reqid));
+    TIMEOUT(fsm_timeout, f, f->timeouttime);
 }
 
 
@@ -753,11 +755,7 @@ fsm_sconfreq(f, retransmit)
  * Used for all packets sent to our peer by this module.
  */
 void
-fsm_sdata(f, code, id, data, datalen)
-    fsm *f;
-    u_char code, id;
-    u_char *data;
-    int datalen;
+fsm_sdata(fsm *f, int code, int id, u_char *data, int datalen)
 {
     u_char *outp;
     int outlen;
@@ -774,7 +772,4 @@ fsm_sdata(f, code, id, data, datalen)
     PUTCHAR(id, outp);
     PUTSHORT(outlen, outp);
     output(f->unit, outpacket_buf, outlen + PPP_HDRLEN);
-
-    FSMDEBUG((LOG_INFO, "fsm_sdata(%s): Sent code %d, id %d.",
-	      PROTO_NAME(f), code, id));
 }
