@@ -39,7 +39,7 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * Copyright (c) 1999-2004 Paul Mackerras. All rights reserved.
+ * Copyright (c) 1999-2020 Paul Mackerras. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -66,8 +66,6 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define RCSID	"$Id: main.c,v 1.156 2008/06/23 11:47:18 paulus Exp $"
-
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -80,7 +78,6 @@
 #include <netdb.h>
 #include <utmp.h>
 #include <pwd.h>
-#include <setjmp.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -121,10 +118,9 @@
 #include "atcp.h"
 #endif
 
-static const char rcsid[] = RCSID;
 
 /* interface vars */
-char ifname[32];		/* Interface name */
+char ifname[MAXIFNAMELEN];	/* Interface name */
 int ifunit;			/* Interface unit number */
 
 struct channel *the_channel;
@@ -159,10 +155,10 @@ TDB_CONTEXT *pppdb;		/* database for storing status etc. */
 
 char db_key[32];
 
-int (*holdoff_hook) __P((void)) = NULL;
-int (*new_phase_hook) __P((int)) = NULL;
-void (*snoop_recv_hook) __P((unsigned char *p, int len)) = NULL;
-void (*snoop_send_hook) __P((unsigned char *p, int len)) = NULL;
+int (*holdoff_hook)(void) = NULL;
+int (*new_phase_hook)(int) = NULL;
+void (*snoop_recv_hook)(unsigned char *p, int len) = NULL;
+void (*snoop_send_hook)(unsigned char *p, int len) = NULL;
 
 static int conn_running;	/* we have a [dis]connector running */
 static int fd_loop;		/* fd for getting demand-dial packets */
@@ -181,7 +177,7 @@ int got_sighup;
 
 static sigset_t signals_handled;
 static int waiting;
-static sigjmp_buf sigjmp;
+static int sigpipe[2];
 
 char **script_env;		/* Env. variable values for scripts */
 int s_env_nalloc;		/* # words avail at script_env */
@@ -218,7 +214,7 @@ bool bundle_terminating;
 struct subprocess {
     pid_t	pid;
     char	*prog;
-    void	(*done) __P((void *));
+    void	(*done)(void *);
     void	*arg;
     int		killable;
     struct subprocess *next;
@@ -228,47 +224,37 @@ static struct subprocess *children;
 
 /* Prototypes for procedures local to this file. */
 
-static void setup_signals __P((void));
-static void create_pidfile __P((int pid));
-static void create_linkpidfile __P((int pid));
-static void cleanup __P((void));
-static void get_input __P((void));
-static void calltimeout __P((void));
-static struct timeval *timeleft __P((struct timeval *));
-static void kill_my_pg __P((int));
-static void hup __P((int));
-static void term __P((int));
-static void chld __P((int));
-static void toggle_debug __P((int));
-static void open_ccp __P((int));
-static void bad_signal __P((int));
-static void holdoff_end __P((void *));
-static void forget_child __P((int pid, int status));
-static int reap_kids __P((void));
-static void childwait_end __P((void *));
+static void setup_signals(void);
+static void create_pidfile(int pid);
+static void create_linkpidfile(int pid);
+static void cleanup(void);
+static void get_input(void);
+static void calltimeout(void);
+static struct timeval *timeleft(struct timeval *);
+static void kill_my_pg(int);
+static void hup(int);
+static void term(int);
+static void chld(int);
+static void toggle_debug(int);
+static void open_ccp(int);
+static void bad_signal(int);
+static void holdoff_end(void *);
+static void forget_child(int pid, int status);
+static int reap_kids(void);
+static void childwait_end(void *);
 
 #ifdef USE_TDB
-static void update_db_entry __P((void));
-static void add_db_key __P((const char *));
-static void delete_db_key __P((const char *));
-static void cleanup_db __P((void));
+static void update_db_entry(void);
+static void add_db_key(const char *);
+static void delete_db_key(const char *);
+static void cleanup_db(void);
 #endif
 
-static void handle_events __P((void));
-void print_link_stats __P((void));
+static void handle_events(void);
+void print_link_stats(void);
 
-extern	char	*ttyname __P((int));
-extern	char	*getlogin __P((void));
-int main __P((int, char *[]));
-
-#ifdef ultrix
-#undef	O_NONBLOCK
-#define	O_NONBLOCK	O_NDELAY
-#endif
-
-#ifdef ULTRIX
-#define setlogmask(x)
-#endif
+extern	char	*getlogin(void);
+int main(int, char *[]);
 
 /*
  * PPP Data Link Layer "protocol" table.
@@ -298,17 +284,8 @@ struct protent *protocols[] = {
     NULL
 };
 
-/*
- * If PPP_DRV_NAME is not defined, use the default "ppp" as the device name.
- */
-#if !defined(PPP_DRV_NAME)
-#define PPP_DRV_NAME	"ppp"
-#endif /* !defined(PPP_DRV_NAME) */
-
 int
-main(argc, argv)
-    int argc;
-    char *argv[];
+main(int argc, char *argv[])
 {
     int i, t;
     char *p;
@@ -316,6 +293,9 @@ main(argc, argv)
     struct protent *protp;
     char numbuf[16];
 
+    strlcpy(path_ipup, _PATH_IPUP, sizeof(path_ipup));
+    strlcpy(path_ipdown, _PATH_IPDOWN, sizeof(path_ipdown));
+
     link_stats_valid = 0;
     new_phase(PHASE_INITIALIZE);
 
@@ -530,7 +510,7 @@ main(argc, argv)
 	    info("Starting link");
 	}
 
-	gettimeofday(&start_time, NULL);
+	get_time(&start_time);
 	script_unsetenv("CONNECT_TIME");
 	script_unsetenv("BYTES_SENT");
 	script_unsetenv("BYTES_RCVD");
@@ -604,22 +584,24 @@ main(argc, argv)
  * handle_events - wait for something to happen and respond to it.
  */
 static void
-handle_events()
+handle_events(void)
 {
     struct timeval timo;
+    unsigned char buf[16];
 
     kill_link = open_ccp_flag = 0;
-    if (sigsetjmp(sigjmp, 1) == 0) {
-	sigprocmask(SIG_BLOCK, &signals_handled, NULL);
-	if (got_sighup || got_sigterm || got_sigusr2 || got_sigchld) {
-	    sigprocmask(SIG_UNBLOCK, &signals_handled, NULL);
-	} else {
-	    waiting = 1;
-	    sigprocmask(SIG_UNBLOCK, &signals_handled, NULL);
-	    wait_input(timeleft(&timo));
-	}
-    }
+
+    /* alert via signal pipe */
+    waiting = 1;
+    /* flush signal pipe */
+    for (; read(sigpipe[0], buf, sizeof(buf)) > 0; );
+    add_fd(sigpipe[0]);
+    /* wait if necessary */
+    if (!(got_sighup || got_sigterm || got_sigusr2 || got_sigchld))
+	wait_input(timeleft(&timo));
     waiting = 0;
+    remove_fd(sigpipe[0]);
+
     calltimeout();
     if (got_sighup) {
 	info("Hangup (SIGHUP)");
@@ -650,10 +632,18 @@ handle_events()
  * setup_signals - initialize signal handling.
  */
 static void
-setup_signals()
+setup_signals(void)
 {
     struct sigaction sa;
 
+    /* create pipe to wake up event handler from signal handler */
+    if (pipe(sigpipe) < 0)
+	fatal("Couldn't create signal pipe: %m");
+    fcntl(sigpipe[0], F_SETFD, fcntl(sigpipe[0], F_GETFD) | FD_CLOEXEC);
+    fcntl(sigpipe[1], F_SETFD, fcntl(sigpipe[1], F_GETFD) | FD_CLOEXEC);
+    fcntl(sigpipe[0], F_SETFL, fcntl(sigpipe[0], F_GETFL) | O_NONBLOCK);
+    fcntl(sigpipe[1], F_SETFL, fcntl(sigpipe[1], F_GETFL) | O_NONBLOCK);
+
     /*
      * Compute mask of all interesting signals and install signal handlers
      * for each.  Only one signal handler may be active at a time.  Therefore,
@@ -734,12 +724,18 @@ setup_signals()
  * unit we are using.
  */
 void
-set_ifunit(iskey)
-    int iskey;
+set_ifunit(int iskey)
 {
-    info("Using interface %s%d", PPP_DRV_NAME, ifunit);
-    slprintf(ifname, sizeof(ifname), "%s%d", PPP_DRV_NAME, ifunit);
+    char ifkey[32];
+
+    if (req_ifname[0] != '\0')
+	slprintf(ifname, sizeof(ifname), "%s", req_ifname);
+    else
+	slprintf(ifname, sizeof(ifname), "%s%d", PPP_DRV_NAME, ifunit);
+    info("Using interface %s", ifname);
     script_setenv("IFNAME", ifname, iskey);
+    slprintf(ifkey, sizeof(ifkey), "%d", ifunit);
+    script_setenv("UNIT", ifkey, iskey);
     if (iskey) {
 	create_pidfile(getpid());	/* write pid to file */
 	create_linkpidfile(getpid());
@@ -750,7 +746,7 @@ set_ifunit(iskey)
  * detach - detach us from the controlling terminal.
  */
 void
-detach()
+detach(void)
 {
     int pid;
     char numbuf[16];
@@ -770,8 +766,7 @@ detach()
 	/* update pid files if they have been written already */
 	if (pidfilename[0])
 	    create_pidfile(pid);
-	if (linkpidfile[0])
-	    create_linkpidfile(pid);
+	create_linkpidfile(pid);
 	exit(0);		/* parent dies */
     }
     setsid();
@@ -795,7 +790,7 @@ detach()
  * reopen_log - (re)open our connection to syslog.
  */
 void
-reopen_log()
+reopen_log(void)
 {
     openlog("pppd", LOG_PID | LOG_NDELAY, LOG_PPP);
     setlogmask(LOG_UPTO(LOG_INFO));
@@ -805,8 +800,7 @@ reopen_log()
  * Create a file containing our process ID.
  */
 static void
-create_pidfile(pid)
-    int pid;
+create_pidfile(int pid)
 {
     FILE *pidfile;
 
@@ -822,8 +816,7 @@ create_pidfile(pid)
 }
 
 void
-create_linkpidfile(pid)
-    int pid;
+create_linkpidfile(int pid)
 {
     FILE *pidfile;
 
@@ -846,7 +839,7 @@ create_linkpidfile(pid)
 /*
  * remove_pidfile - remove our pid files
  */
-void remove_pidfiles()
+void remove_pidfiles(void)
 {
     if (pidfilename[0] != 0 && unlink(pidfilename) < 0 && errno != ENOENT)
 	warn("unable to delete pid file %s: %m", pidfilename);
@@ -860,8 +853,7 @@ void remove_pidfiles()
  * holdoff_end - called via a timeout when the holdoff period ends.
  */
 static void
-holdoff_end(arg)
-    void *arg;
+holdoff_end(void *arg)
 {
     new_phase(PHASE_DORMANT);
 }
@@ -1005,8 +997,7 @@ struct protocol_list {
  * protocol_name - find a name for a PPP protocol.
  */
 const char *
-protocol_name(proto)
-    int proto;
+protocol_name(int proto)
 {
     struct protocol_list *lp;
 
@@ -1020,7 +1011,7 @@ protocol_name(proto)
  * get_input - called when incoming data is available.
  */
 static void
-get_input()
+get_input(void)
 {
     int len, i;
     u_char *p;
@@ -1112,10 +1103,7 @@ get_input()
  * itself), otherwise 0.
  */
 int
-ppp_send_config(unit, mtu, accm, pcomp, accomp)
-    int unit, mtu;
-    u_int32_t accm;
-    int pcomp, accomp;
+ppp_send_config(int unit, int mtu, u_int32_t accm, int pcomp, int accomp)
 {
 	int errs;
 
@@ -1133,10 +1121,7 @@ ppp_send_config(unit, mtu, accm, pcomp, accomp)
  * itself), otherwise 0.
  */
 int
-ppp_recv_config(unit, mru, accm, pcomp, accomp)
-    int unit, mru;
-    u_int32_t accm;
-    int pcomp, accomp;
+ppp_recv_config(int unit, int mru, u_int32_t accm, int pcomp, int accomp)
 {
 	int errs;
 
@@ -1151,8 +1136,7 @@ ppp_recv_config(unit, mru, accm, pcomp, accomp)
  * new_phase - signal the start of a new phase of pppd's operation.
  */
 void
-new_phase(p)
-    int p;
+new_phase(int p)
 {
     phase = p;
     if (new_phase_hook)
@@ -1164,8 +1148,7 @@ new_phase(p)
  * die - clean up state and exit with the specified status.
  */
 void
-die(status)
-    int status;
+die(int status)
 {
     if (!doing_multilink || multilink_master)
 	print_link_stats();
@@ -1180,7 +1163,7 @@ die(status)
  */
 /* ARGSUSED */
 static void
-cleanup()
+cleanup(void)
 {
     sys_cleanup();
 
@@ -1198,7 +1181,7 @@ cleanup()
 }
 
 void
-print_link_stats()
+print_link_stats(void)
 {
     /*
      * Print connect time and statistics.
@@ -1216,26 +1199,24 @@ print_link_stats()
  * reset_link_stats - "reset" stats when link goes up.
  */
 void
-reset_link_stats(u)
-    int u;
+reset_link_stats(int u)
 {
     if (!get_ppp_stats(u, &old_link_stats))
 	return;
-    gettimeofday(&start_time, NULL);
+    get_time(&start_time);
 }
 
 /*
  * update_link_stats - get stats at link termination.
  */
 void
-update_link_stats(u)
-    int u;
+update_link_stats(int u)
 {
     struct timeval now;
     char numbuf[32];
 
     if (!get_ppp_stats(u, &link_stats)
-	|| gettimeofday(&now, NULL) < 0)
+	|| get_time(&now) < 0)
 	return;
     link_connect_time = now.tv_sec - start_time.tv_sec;
     link_stats_valid = 1;
@@ -1257,7 +1238,7 @@ update_link_stats(u)
 struct	callout {
     struct timeval	c_time;		/* time at which to call routine */
     void		*c_arg;		/* argument to routine */
-    void		(*c_func) __P((void *)); /* routine */
+    void		(*c_func)(void *); /* routine */
     struct		callout *c_next;
 };
 
@@ -1268,10 +1249,7 @@ static struct timeval timenow;		/* Current time */
  * timeout - Schedule a timeout.
  */
 void
-timeout(func, arg, secs, usecs)
-    void (*func) __P((void *));
-    void *arg;
-    int secs, usecs;
+timeout(void (*func)(void *), void *arg, int secs, int usecs)
 {
     struct callout *newp, *p, **pp;
 
@@ -1282,7 +1260,7 @@ timeout(func, arg, secs, usecs)
 	fatal("Out of memory in timeout()!");
     newp->c_arg = arg;
     newp->c_func = func;
-    gettimeofday(&timenow, NULL);
+    get_time(&timenow);
     newp->c_time.tv_sec = timenow.tv_sec + secs;
     newp->c_time.tv_usec = timenow.tv_usec + usecs;
     if (newp->c_time.tv_usec >= 1000000) {
@@ -1307,9 +1285,7 @@ timeout(func, arg, secs, usecs)
  * untimeout - Unschedule a timeout.
  */
 void
-untimeout(func, arg)
-    void (*func) __P((void *));
-    void *arg;
+untimeout(void (*func)(void *), void *arg)
 {
     struct callout **copp, *freep;
 
@@ -1329,14 +1305,14 @@ untimeout(func, arg)
  * calltimeout - Call any timeout routines which are now due.
  */
 static void
-calltimeout()
+calltimeout(void)
 {
     struct callout *p;
 
     while (callout != NULL) {
 	p = callout;
 
-	if (gettimeofday(&timenow, NULL) < 0)
+	if (get_time(&timenow) < 0)
 	    fatal("Failed to get time of day: %m");
 	if (!(p->c_time.tv_sec < timenow.tv_sec
 	      || (p->c_time.tv_sec == timenow.tv_sec
@@ -1355,13 +1331,12 @@ calltimeout()
  * timeleft - return the length of time until the next timeout is due.
  */
 static struct timeval *
-timeleft(tvp)
-    struct timeval *tvp;
+timeleft(struct timeval *tvp)
 {
     if (callout == NULL)
 	return NULL;
 
-    gettimeofday(&timenow, NULL);
+    get_time(&timenow);
     tvp->tv_sec = callout->c_time.tv_sec - timenow.tv_sec;
     tvp->tv_usec = callout->c_time.tv_usec - timenow.tv_usec;
     if (tvp->tv_usec < 0) {
@@ -1380,8 +1355,7 @@ timeleft(tvp)
  * We assume that sig is currently blocked.
  */
 static void
-kill_my_pg(sig)
-    int sig;
+kill_my_pg(int sig)
 {
     struct sigaction act, oldact;
     struct subprocess *chp;
@@ -1427,8 +1401,7 @@ kill_my_pg(sig)
  * signal, we just take the link down.
  */
 static void
-hup(sig)
-    int sig;
+hup(int sig)
 {
     /* can't log a message here, it can deadlock */
     got_sighup = 1;
@@ -1437,7 +1410,7 @@ hup(sig)
 	kill_my_pg(sig);
     notify(sigreceived, sig);
     if (waiting)
-	siglongjmp(sigjmp, 1);
+	write(sigpipe[1], &sig, sizeof(sig));
 }
 
 
@@ -1448,8 +1421,7 @@ hup(sig)
  */
 /*ARGSUSED*/
 static void
-term(sig)
-    int sig;
+term(int sig)
 {
     /* can't log a message here, it can deadlock */
     got_sigterm = sig;
@@ -1458,7 +1430,7 @@ term(sig)
 	kill_my_pg(sig);
     notify(sigreceived, sig);
     if (waiting)
-	siglongjmp(sigjmp, 1);
+	write(sigpipe[1], &sig, sizeof(sig));
 }
 
 
@@ -1467,12 +1439,11 @@ term(sig)
  * Sets a flag so we will call reap_kids in the mainline.
  */
 static void
-chld(sig)
-    int sig;
+chld(int sig)
 {
     got_sigchld = 1;
     if (waiting)
-	siglongjmp(sigjmp, 1);
+	write(sigpipe[1], &sig, sizeof(sig));
 }
 
 
@@ -1483,8 +1454,7 @@ chld(sig)
  */
 /*ARGSUSED*/
 static void
-toggle_debug(sig)
-    int sig;
+toggle_debug(int sig)
 {
     debug = !debug;
     if (debug) {
@@ -1502,12 +1472,11 @@ toggle_debug(sig)
  */
 /*ARGSUSED*/
 static void
-open_ccp(sig)
-    int sig;
+open_ccp(int sig)
 {
     got_sigusr2 = 1;
     if (waiting)
-	siglongjmp(sigjmp, 1);
+	write(sigpipe[1], &sig, sizeof(sig));
 }
 
 
@@ -1515,8 +1484,7 @@ open_ccp(sig)
  * bad_signal - We've caught a fatal signal.  Clean up state and exit.
  */
 static void
-bad_signal(sig)
-    int sig;
+bad_signal(int sig)
 {
     static int crashed = 0;
 
@@ -1571,7 +1539,8 @@ safe_fork(int infd, int outfd, int errfd)
 	/* Executing in the child */
 	sys_close();
 #ifdef USE_TDB
-	tdb_close(pppdb);
+	if (pppdb != NULL)
+		tdb_close(pppdb);
 #endif
 
 	/* make sure infd, outfd and errfd won't get tromped on below */
@@ -1616,9 +1585,7 @@ safe_fork(int infd, int outfd, int errfd)
 }
 
 static bool
-add_script_env(pos, newstring)
-    int pos;
-    char *newstring;
+add_script_env(int pos, char *newstring)
 {
     if (pos + 1 >= s_env_nalloc) {
 	int new_n = pos + 17;
@@ -1636,8 +1603,7 @@ add_script_env(pos, newstring)
 }
 
 static void
-remove_script_env(pos)
-    int pos;
+remove_script_env(int pos)
 {
     free(script_env[pos] - 1);
     while ((script_env[pos] = script_env[pos + 1]) != NULL)
@@ -1649,7 +1615,7 @@ remove_script_env(pos)
  * and update the system environment.
  */
 static void
-update_system_environment()
+update_system_environment(void)
 {
     struct userenv *uep;
 
@@ -1667,10 +1633,7 @@ update_system_environment()
  * stderr gets connected to the log fd or to the _PATH_CONNERRS file.
  */
 int
-device_script(program, in, out, dont_wait)
-    char *program;
-    int in, out;
-    int dont_wait;
+device_script(char *program, int in, int out, int dont_wait)
 {
     int pid;
     int status = -1;
@@ -1679,7 +1642,7 @@ device_script(program, in, out, dont_wait)
     if (log_to_fd >= 0)
 	errfd = log_to_fd;
     else
-	errfd = open(_PATH_CONNERRS, O_WRONLY | O_APPEND | O_CREAT, 0600);
+	errfd = open(_PATH_CONNERRS, O_WRONLY | O_APPEND | O_CREAT, 0644);
 
     ++conn_running;
     pid = safe_fork(in, out, errfd);
@@ -1732,7 +1695,7 @@ device_script(program, in, out, dont_wait)
  * script_unsetenv() safely after this routine is run.
  */
 static void
-update_script_environment()
+update_script_environment(void)
 {
     struct userenv *uep;
 
@@ -1756,7 +1719,7 @@ update_script_environment()
 		script_env[i] = newstring;
 	    else
 		add_script_env(i, newstring);
-	} else {
+	} else if (p != NULL) {
 	    remove_script_env(i);
 	}
     }
@@ -1773,13 +1736,7 @@ update_script_environment()
  * reap_kids) iff the return value is > 0.
  */
 pid_t
-run_program(prog, args, must_exist, done, arg, wait)
-    char *prog;
-    char **args;
-    int must_exist;
-    void (*done) __P((void *));
-    void *arg;
-    int wait;
+run_program(char *prog, char **args, int must_exist, void (*done)(void *), void *arg, int wait)
 {
     int pid, status;
     struct stat sbuf;
@@ -1850,12 +1807,7 @@ run_program(prog, args, must_exist, done, arg, wait)
  * to use.
  */
 void
-record_child(pid, prog, done, arg, killable)
-    int pid;
-    char *prog;
-    void (*done) __P((void *));
-    void *arg;
-    int killable;
+record_child(int pid, char *prog, void (*done)(void *), void *arg, int killable)
 {
     struct subprocess *chp;
 
@@ -1880,8 +1832,7 @@ record_child(pid, prog, done, arg, killable)
  * exit, send them all a SIGTERM.
  */
 static void
-childwait_end(arg)
-    void *arg;
+childwait_end(void *arg)
 {
     struct subprocess *chp;
 
@@ -1897,8 +1848,7 @@ childwait_end(arg)
  * forget_child - clean up after a dead child
  */
 static void
-forget_child(pid, status)
-    int pid, status;
+forget_child(int pid, int status)
 {
     struct subprocess *chp, **prevp;
 
@@ -1927,7 +1877,7 @@ forget_child(pid, status)
  * and log a message for abnormal terminations.
  */
 static int
-reap_kids()
+reap_kids(void)
 {
     int pid, status;
 
@@ -1949,10 +1899,7 @@ reap_kids()
  * add_notifier - add a new function to be called when something happens.
  */
 void
-add_notifier(notif, func, arg)
-    struct notifier **notif;
-    notify_func func;
-    void *arg;
+add_notifier(struct notifier **notif, notify_func func, void *arg)
 {
     struct notifier *np;
 
@@ -1970,10 +1917,7 @@ add_notifier(notif, func, arg)
  * be called when something happens.
  */
 void
-remove_notifier(notif, func, arg)
-    struct notifier **notif;
-    notify_func func;
-    void *arg;
+remove_notifier(struct notifier **notif, notify_func func, void *arg)
 {
     struct notifier *np;
 
@@ -1990,9 +1934,7 @@ remove_notifier(notif, func, arg)
  * notify - call a set of functions registered with add_notifier.
  */
 void
-notify(notif, val)
-    struct notifier *notif;
-    int val;
+notify(struct notifier *notif, int val)
 {
     struct notifier *np;
 
@@ -2006,8 +1948,7 @@ notify(notif, val)
  * novm - log an error message saying we ran out of memory, and die.
  */
 void
-novm(msg)
-    char *msg;
+novm(char *msg)
 {
     fatal("Virtual memory exhausted allocating %s\n", msg);
 }
@@ -2017,9 +1958,7 @@ novm(msg)
  * for scripts that we run (e.g. ip-up, auth-up, etc.)
  */
 void
-script_setenv(var, value, iskey)
-    char *var, *value;
-    int iskey;
+script_setenv(char *var, char *value, int iskey)
 {
     size_t varl = strlen(var);
     size_t vl = varl + strlen(value) + 2;
@@ -2080,8 +2019,7 @@ script_setenv(var, value, iskey)
  * for scripts.
  */
 void
-script_unsetenv(var)
-    char *var;
+script_unsetenv(char *var)
 {
     int vl = strlen(var);
     int i;
@@ -2115,7 +2053,7 @@ script_unsetenv(var)
  * lock_db - get an exclusive lock on the TDB database.
  * Used to ensure atomicity of various lookup/modify operations.
  */
-void lock_db()
+void lock_db(void)
 {
 #ifdef USE_TDB
 	TDB_DATA key;
@@ -2129,7 +2067,7 @@ void lock_db()
 /*
  * unlock_db - remove the exclusive lock obtained by lock_db.
  */
-void unlock_db()
+void unlock_db(void)
 {
 #ifdef USE_TDB
 	TDB_DATA key;
@@ -2145,7 +2083,7 @@ void unlock_db()
  * update_db_entry - update our entry in the database.
  */
 static void
-update_db_entry()
+update_db_entry(void)
 {
     TDB_DATA key, dbuf;
     int vlen, i;
@@ -2179,8 +2117,7 @@ update_db_entry()
  * add_db_key - add a key that we can use to look up our database entry.
  */
 static void
-add_db_key(str)
-    const char *str;
+add_db_key(const char *str)
 {
     TDB_DATA key, dbuf;
 
@@ -2196,8 +2133,7 @@ add_db_key(str)
  * delete_db_key - delete a key for looking up our database entry.
  */
 static void
-delete_db_key(str)
-    const char *str;
+delete_db_key(const char *str)
 {
     TDB_DATA key;
 
@@ -2210,7 +2146,7 @@ delete_db_key(str)
  * cleanup_db - delete all the entries we put in the database.
  */
 static void
-cleanup_db()
+cleanup_db(void)
 {
     TDB_DATA key;
     int i;
