@@ -40,8 +40,6 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define RCSID	"$Id: upap.c,v 1.30 2005/07/13 10:41:58 paulus Exp $"
-
 /*
  * TODO:
  */
@@ -52,7 +50,6 @@
 #include "pppd.h"
 #include "upap.h"
 
-static const char rcsid[] = RCSID;
 
 static bool hide_password = 1;
 
@@ -78,13 +75,13 @@ static option_t pap_option_list[] = {
 /*
  * Protocol entry points.
  */
-static void upap_init __P((int));
-static void upap_lowerup __P((int));
-static void upap_lowerdown __P((int));
-static void upap_input __P((int, u_char *, int));
-static void upap_protrej __P((int));
-static int  upap_printpkt __P((u_char *, int,
-			       void (*) __P((void *, char *, ...)), void *));
+static void upap_init(int);
+static void upap_lowerup(int);
+static void upap_lowerdown(int);
+static void upap_input(int, u_char *, int);
+static void upap_protrej(int);
+static int  upap_printpkt(u_char *, int,
+			  void (*)(void *, char *, ...), void *);
 
 struct protent pap_protent = {
     PPP_PAP,
@@ -108,21 +105,20 @@ struct protent pap_protent = {
 
 upap_state upap[NUM_PPP];		/* UPAP state; one for each unit */
 
-static void upap_timeout __P((void *));
-static void upap_reqtimeout __P((void *));
-static void upap_rauthreq __P((upap_state *, u_char *, int, int));
-static void upap_rauthack __P((upap_state *, u_char *, int, int));
-static void upap_rauthnak __P((upap_state *, u_char *, int, int));
-static void upap_sauthreq __P((upap_state *));
-static void upap_sresp __P((upap_state *, int, int, char *, int));
+static void upap_timeout(void *);
+static void upap_reqtimeout(void *);
+static void upap_rauthreq(upap_state *, u_char *, int, int);
+static void upap_rauthack(upap_state *, u_char *, int, int);
+static void upap_rauthnak(upap_state *, u_char *, int, int);
+static void upap_sauthreq(upap_state *);
+static void upap_sresp(upap_state *, int, int, char *, int);
 
 
 /*
  * upap_init - Initialize a UPAP unit.
  */
 static void
-upap_init(unit)
-    int unit;
+upap_init(int unit)
 {
     upap_state *u = &upap[unit];
 
@@ -146,9 +142,7 @@ upap_init(unit)
  * Set new state and send authenticate's.
  */
 void
-upap_authwithpeer(unit, user, password)
-    int unit;
-    char *user, *password;
+upap_authwithpeer(int unit, char *user, char *password)
 {
     upap_state *u = &upap[unit];
 
@@ -176,8 +170,7 @@ upap_authwithpeer(unit, user, password)
  * Set new state.
  */
 void
-upap_authpeer(unit)
-    int unit;
+upap_authpeer(int unit)
 {
     upap_state *u = &upap[unit];
 
@@ -198,8 +191,7 @@ upap_authpeer(unit)
  * upap_timeout - Retransmission timer for sending auth-reqs expired.
  */
 static void
-upap_timeout(arg)
-    void *arg;
+upap_timeout(void *arg)
 {
     upap_state *u = (upap_state *) arg;
 
@@ -222,8 +214,7 @@ upap_timeout(arg)
  * upap_reqtimeout - Give up waiting for the peer to send an auth-req.
  */
 static void
-upap_reqtimeout(arg)
-    void *arg;
+upap_reqtimeout(void *arg)
 {
     upap_state *u = (upap_state *) arg;
 
@@ -241,8 +232,7 @@ upap_reqtimeout(arg)
  * Start authenticating if pending.
  */
 static void
-upap_lowerup(unit)
-    int unit;
+upap_lowerup(int unit)
 {
     upap_state *u = &upap[unit];
 
@@ -268,8 +258,7 @@ upap_lowerup(unit)
  * Cancel all timeouts.
  */
 static void
-upap_lowerdown(unit)
-    int unit;
+upap_lowerdown(int unit)
 {
     upap_state *u = &upap[unit];
 
@@ -289,8 +278,7 @@ upap_lowerdown(unit)
  * This shouldn't happen.  In any case, pretend lower layer went down.
  */
 static void
-upap_protrej(unit)
-    int unit;
+upap_protrej(int unit)
 {
     upap_state *u = &upap[unit];
 
@@ -310,10 +298,7 @@ upap_protrej(unit)
  * upap_input - Input UPAP packet.
  */
 static void
-upap_input(unit, inpacket, l)
-    int unit;
-    u_char *inpacket;
-    int l;
+upap_input(int unit, u_char *inpacket, int l)
 {
     upap_state *u = &upap[unit];
     u_char *inp;
@@ -368,11 +353,7 @@ upap_input(unit, inpacket, l)
  * upap_rauth - Receive Authenticate.
  */
 static void
-upap_rauthreq(u, inp, id, len)
-    upap_state *u;
-    u_char *inp;
-    int id;
-    int len;
+upap_rauthreq(upap_state *u, u_char *inp, int id, int len)
 {
     u_char ruserlen, rpasswdlen;
     char *ruser, *rpasswd;
@@ -466,11 +447,7 @@ upap_rauthreq(u, inp, id, len)
  * upap_rauthack - Receive Authenticate-Ack.
  */
 static void
-upap_rauthack(u, inp, id, len)
-    upap_state *u;
-    u_char *inp;
-    int id;
-    int len;
+upap_rauthack(upap_state *u, u_char *inp, int id, int len)
 {
     u_char msglen;
     char *msg;
@@ -506,11 +483,7 @@ upap_rauthack(u, inp, id, len)
  * upap_rauthnak - Receive Authenticate-Nak.
  */
 static void
-upap_rauthnak(u, inp, id, len)
-    upap_state *u;
-    u_char *inp;
-    int id;
-    int len;
+upap_rauthnak(upap_state *u, u_char *inp, int id, int len)
 {
     u_char msglen;
     char *msg;
@@ -547,8 +520,7 @@ upap_rauthnak(u, inp, id, len)
  * upap_sauthreq - Send an Authenticate-Request.
  */
 static void
-upap_sauthreq(u)
-    upap_state *u;
+upap_sauthreq(upap_state *u)
 {
     u_char *outp;
     int outlen;
@@ -580,11 +552,7 @@ upap_sauthreq(u)
  * upap_sresp - Send a response (ack or nak).
  */
 static void
-upap_sresp(u, code, id, msg, msglen)
-    upap_state *u;
-    u_char code, id;
-    char *msg;
-    int msglen;
+upap_sresp(upap_state *u, int code, int id, char *msg, int msglen)
 {
     u_char *outp;
     int outlen;
@@ -609,11 +577,7 @@ static char *upap_codenames[] = {
 };
 
 static int
-upap_printpkt(p, plen, printer, arg)
-    u_char *p;
-    int plen;
-    void (*printer) __P((void *, char *, ...));
-    void *arg;
+upap_printpkt(u_char *p, int plen, void (*printer)(void *, char *, ...), void *arg)
 {
     int code, id, len;
     int mlen, ulen, wlen;
